=== modified file 'ChangeLog'
--- ChangeLog	2010-08-12 13:49:45 +0000
+++ ChangeLog	2010-11-10 23:23:22 +0000
@@ -1,3 +1,11 @@
+2010-11-10  Colin Watson  <cjwatson@ubuntu.com>
+
+	* dbus/com.ubuntu.Upstart.Instance.xml (StateChanged): New signal.
+	* init/job.c (job_change_state): Emit StateChanged signal after
+	changing state.
+	* init/tests/test_job.c (test_change_state): Test for this.
+	* NEWS: Update.
+
 2010-08-12  Scott James Remnant  <scott@netsplit.com>
 
 	* init/job_process.c (job_process_spawn): We can fail to open the

=== modified file 'NEWS'
--- NEWS	2010-04-27 19:48:44 +0000
+++ NEWS	2010-11-10 23:23:22 +0000
@@ -1,5 +1,8 @@
 0.6.7  xxxx-xx-xx
 
+	* Upstart now emits a "StateChanged" D-Bus signal immediately after
+	  changing the "state" property of an instance.
+
 0.6.6  2010-04-27  "No, she's dead, this is her son"
 
 	* Upstart will automatically mount /proc and /sys on boot if they

=== modified file 'dbus/com.ubuntu.Upstart.Instance.xml'
--- dbus/com.ubuntu.Upstart.Instance.xml	2009-07-03 16:38:02 +0000
+++ dbus/com.ubuntu.Upstart.Instance.xml	2010-11-10 23:23:22 +0000
@@ -36,6 +36,10 @@
       <arg name="wait" type="b" direction="in" />
     </method>
 
+    <signal name="StateChanged">
+      <arg name="state" type="s" />
+    </signal>
+
     <!-- Basic information about an Instance -->
     <property name="name" type="s" access="read" />
     <property name="goal" type="s" access="read" />

=== modified file 'init/job.c'
--- init/job.c	2009-07-21 11:27:55 +0000
+++ init/job.c	2010-11-10 23:23:22 +0000
@@ -288,6 +288,15 @@
 		old_state = job->state;
 		job->state = state;
 
+		NIH_LIST_FOREACH (control_conns, iter) {
+			NihListEntry   *entry = (NihListEntry *)iter;
+			DBusConnection *conn = (DBusConnection *)entry->data;
+
+			NIH_ZERO (job_emit_state_changed (
+					conn, job->path,
+					job_state_name (job->state)));
+		}
+
 		/* Perform whatever action is necessary to enter the new
 		 * state, such as executing a process or emitting an event.
 		 */

=== modified file 'init/tests/test_job.c'
--- init/tests/test_job.c	2009-07-09 11:01:53 +0000
+++ init/tests/test_job.c	2010-11-10 23:23:22 +0000
@@ -615,7 +615,7 @@
 	DBusError        dbus_error;
 	DBusConnection  *conn, *client_conn;
 	DBusMessage     *message;
-	char            *path, *job_path = NULL;
+	char            *path, *job_path = NULL, *state;
 	int              status;
 
 	TEST_FUNCTION ("job_change_state");
@@ -653,9 +653,25 @@
 	 * environment from start_env.
 	 */
 	TEST_FEATURE ("waiting to starting");
+	dbus_error_init (&dbus_error);
+
+	TEST_DBUS (dbus_pid);
+	TEST_DBUS_OPEN (conn);
+	TEST_DBUS_OPEN (client_conn);
+
+	dbus_bus_add_match (client_conn, "type='signal'", &dbus_error);
+	assert (! dbus_error_is_set (&dbus_error));
+
+	control_init ();
+
+	entry = nih_list_entry_new (NULL);
+	entry->data = conn;
+	nih_list_add (control_conns, &entry->entry);
+
 	TEST_ALLOC_FAIL {
 		TEST_ALLOC_SAFE {
 			job = job_new (class, "");
+			job_path = nih_strdup (NULL, job->path);
 
 			assert (nih_str_array_add (&(job->start_env), job,
 						   NULL, "FOO=BAR"));
@@ -667,6 +683,14 @@
 			nih_list_add (&job->blocking, &blocked->entry);
 		}
 
+		dbus_connection_flush (conn);
+
+		TEST_DBUS_MESSAGE (client_conn, message);
+		TEST_TRUE (dbus_message_is_signal (message, DBUS_INTERFACE_UPSTART_JOB,
+						   "InstanceAdded"));
+
+		dbus_message_unref (message);
+
 		job->goal = JOB_START;
 		job->state = JOB_WAITING;
 
@@ -686,6 +710,22 @@
 		TEST_EQ (job->goal, JOB_START);
 		TEST_EQ (job->state, JOB_STARTING);
 
+		dbus_connection_flush (conn);
+
+		TEST_DBUS_MESSAGE (client_conn, message);
+		TEST_TRUE (dbus_message_is_signal (message, DBUS_INTERFACE_UPSTART_INSTANCE,
+						   "StateChanged"));
+
+		TEST_EQ_STR (dbus_message_get_path (message), job_path);
+
+		TEST_TRUE (dbus_message_get_args (message, NULL,
+						  DBUS_TYPE_STRING, &state,
+						  DBUS_TYPE_INVALID));
+
+		TEST_EQ_STR (state, "starting");
+
+		dbus_message_unref (message);
+
 		TEST_EQ (cause->blockers, 1);
 		TEST_EQ (cause->failed, FALSE);
 
@@ -728,6 +768,14 @@
 		nih_free (job);
 	}
 
+	nih_free (entry);
+
+	TEST_DBUS_CLOSE (conn);
+	TEST_DBUS_CLOSE (client_conn);
+	TEST_DBUS_END (dbus_pid);
+
+	dbus_shutdown ();
+
 
 	/* Check that a named instance of a job can move from waiting to
 	 * starting, and that the instance name is included in the event
@@ -3372,6 +3420,18 @@
 		dbus_connection_flush (conn);
 
 		TEST_DBUS_MESSAGE (client_conn, message);
+		TEST_TRUE (dbus_message_is_signal (message, DBUS_INTERFACE_UPSTART_INSTANCE,
+						   "StateChanged"));
+
+		TEST_EQ_STR (dbus_message_get_path (message), job_path);
+
+		TEST_TRUE (dbus_message_get_args (message, NULL,
+						  DBUS_TYPE_STRING, &state,
+						  DBUS_TYPE_INVALID));
+
+		TEST_EQ_STR (state, "waiting");
+
+		TEST_DBUS_MESSAGE (client_conn, message);
 		TEST_TRUE (dbus_message_is_signal (message, DBUS_INTERFACE_UPSTART_JOB,
 						   "InstanceRemoved"));
 

